@@ -1,3 +1,20 @@
+/// Utility macro to return the name of the current function.
+#[doc(hidden)]
+#[macro_export]
+macro_rules! _function_name {
+    () => {{
+        fn f() {}
+        fn type_name_of_val<T>(_: T) -> &'static str {
+            std::any::type_name::<T>()
+        }
+        let mut name = type_name_of_val(f).strip_suffix("::f").unwrap_or("");
+        while let Some(rest) = name.strip_suffix("::{{closure}}") {
+            name = rest;
+        }
+        name
+    }};
+}
+
 /// Asserts a `Serialize` snapshot in CSV format.
 ///
 /// **Feature:** `csv` (disabled by default)
@@ -8,8 +25,8 @@
 ///
 /// Example:
 ///
-/// ```no_run,ignore
-/// assert_csv_snapshot!(vec[1, 2, 3]);
+/// ```no_run
+/// insta::assert_csv_snapshot!(vec![1, 2, 3]);
 /// ```
 ///
 /// The third argument to the macro can be an object expression for redaction.
@@ -18,6 +35,7 @@
 ///
 /// The snapshot name is optional but can be provided as first argument.
 #[cfg(feature = "csv")]
+#[cfg_attr(docsrs, doc(cfg(feature = "csv")))]
 #[macro_export]
 macro_rules! assert_csv_snapshot {
     ($value:expr, @$snapshot:literal) => {{
@@ -51,8 +69,8 @@ macro_rules! assert_csv_snapshot {
 ///
 /// Example:
 ///
-/// ```no_run,ignore
-/// assert_toml_snapshot!(vec[1, 2, 3]);
+/// ```no_run
+/// insta::assert_toml_snapshot!(vec![1, 2, 3]);
 /// ```
 ///
 /// The third argument to the macro can be an object expression for redaction.
@@ -61,6 +79,7 @@ macro_rules! assert_csv_snapshot {
 ///
 /// The snapshot name is optional but can be provided as first argument.
 #[cfg(feature = "toml")]
+#[cfg_attr(docsrs, doc(cfg(feature = "toml")))]
 #[macro_export]
 macro_rules! assert_toml_snapshot {
     ($value:expr, @$snapshot:literal) => {{
@@ -85,12 +104,14 @@ macro_rules! assert_toml_snapshot {
 
 /// Asserts a `Serialize` snapshot in YAML format.
 ///
+/// **Feature:** `yaml`
+///
 /// The value needs to implement the `serde::Serialize` trait and the snapshot
 /// will be serialized in YAML format.  This does mean that unlike the debug
 /// snapshot variant the type of the value does not appear in the output.
 /// You can however use the `assert_ron_snapshot!` macro to dump out
 /// the value in [RON](https://github.com/ron-rs/ron/) format which retains some
-/// type information for more accurate comparisions.
+/// type information for more accurate comparisons.
 ///
 /// Example:
 ///
@@ -125,6 +146,8 @@ macro_rules! assert_toml_snapshot {
 /// just use an empty string (`@""`).
 ///
 /// The snapshot name is optional but can be provided as first argument.
+#[cfg(feature = "yaml")]
+#[cfg_attr(docsrs, doc(cfg(feature = "yaml")))]
 #[macro_export]
 macro_rules! assert_yaml_snapshot {
     ($value:expr, @$snapshot:literal) => {{
@@ -153,7 +176,7 @@ macro_rules! assert_yaml_snapshot {
 ///
 /// This works exactly like [`assert_yaml_snapshot!`]
 /// but serializes in [RON](https://github.com/ron-rs/ron/) format instead of
-/// YAML which retains some type information for more accurate comparisions.
+/// YAML which retains some type information for more accurate comparisons.
 ///
 /// Example:
 ///
@@ -168,6 +191,7 @@ macro_rules! assert_yaml_snapshot {
 ///
 /// The snapshot name is optional but can be provided as first argument.
 #[cfg(feature = "ron")]
+#[cfg_attr(docsrs, doc(cfg(feature = "ron")))]
 #[macro_export]
 macro_rules! assert_ron_snapshot {
     ($value:expr, @$snapshot:literal) => {{
@@ -192,6 +216,8 @@ macro_rules! assert_ron_snapshot {
 
 /// Asserts a `Serialize` snapshot in JSON format.
 ///
+/// **Feature:** `json`
+///
 /// This works exactly like [`assert_yaml_snapshot!`] but serializes in JSON format.
 /// This is normally not recommended because it makes diffs less reliable, but it can
 /// be useful for certain specialized situations.
@@ -208,6 +234,8 @@ macro_rules! assert_ron_snapshot {
 /// about redactions refer to the [redactions feature in the guide](https://insta.rs/docs/redactions/).
 ///
 /// The snapshot name is optional but can be provided as first argument.
+#[cfg(feature = "json")]
+#[cfg_attr(docsrs, doc(cfg(feature = "json")))]
 #[macro_export]
 macro_rules! assert_json_snapshot {
     ($value:expr, @$snapshot:literal) => {{
@@ -230,6 +258,51 @@ macro_rules! assert_json_snapshot {
     }};
 }
 
+/// Asserts a `Serialize` snapshot in compact JSON format.
+///
+/// **Feature:** `json`
+///
+/// This works exactly like [`assert_json_snapshot!`] but serializes into a single
+/// line for as long as the output is less than 120 characters.  This can be useful
+/// in cases where you are working with small result outputs but comes at the cost
+/// of slightly worse diffing behavior.
+///
+/// Example:
+///
+/// ```no_run
+/// # use insta::*;
+/// assert_compact_json_snapshot!(vec![1, 2, 3]);
+/// ```
+///
+/// The third argument to the macro can be an object expression for redaction.
+/// It's in the form `{ selector => replacement }`.  For more information
+/// about redactions refer to the [redactions feature in the guide](https://insta.rs/docs/redactions/).
+///
+/// The snapshot name is optional but can be provided as first argument.
+#[cfg(feature = "json")]
+#[cfg_attr(docsrs, doc(cfg(feature = "json")))]
+#[macro_export]
+macro_rules! assert_compact_json_snapshot {
+    ($value:expr, @$snapshot:literal) => {{
+        $crate::_assert_serialized_snapshot!($value, JsonCompact, @$snapshot);
+    }};
+    ($value:expr, {$($k:expr => $v:expr),*$(,)?}, @$snapshot:literal) => {{
+        $crate::_assert_serialized_snapshot!($value, {$($k => $v),*}, JsonCompact, @$snapshot);
+    }};
+    ($value:expr, {$($k:expr => $v:expr),*$(,)?}) => {{
+        $crate::_assert_serialized_snapshot!($crate::_macro_support::AutoName, $value, {$($k => $v),*}, JsonCompact);
+    }};
+    ($name:expr, $value:expr) => {{
+        $crate::_assert_serialized_snapshot!(Some($name), $value, JsonCompact);
+    }};
+    ($name:expr, $value:expr, {$($k:expr => $v:expr),*$(,)?}) => {{
+        $crate::_assert_serialized_snapshot!(Some($name), $value, {$($k => $v),*}, JsonCompact);
+    }};
+    ($value:expr) => {{
+        $crate::_assert_serialized_snapshot!($crate::_macro_support::AutoName, $value, JsonCompact);
+    }};
+}
+
 #[doc(hidden)]
 #[macro_export]
 macro_rules! _assert_serialized_snapshot {
@@ -378,19 +451,20 @@ macro_rules! assert_snapshot {
     ($name:expr, $value:expr) => {
         $crate::assert_snapshot!($name, $value, stringify!($value))
     };
-    ($name:expr, $value:expr, $debug_expr:expr) => {
+    ($name:expr, $value:expr, $debug_expr:expr) => {{
         $crate::_macro_support::assert_snapshot(
             // Creates a ReferenceValue::Named variant
             $name.into(),
             &$value,
             env!("CARGO_MANIFEST_DIR"),
+            $crate::_function_name!(),
             module_path!(),
             file!(),
             line!(),
             $debug_expr,
         )
-        .unwrap();
-    };
+        .unwrap()
+    }};
     ($value:expr) => {
         $crate::assert_snapshot!($crate::_macro_support::AutoName, $value, stringify!($value))
     };
@@ -398,22 +472,39 @@ macro_rules! assert_snapshot {
 
 /// Settings configuration macro.
 ///
-/// This macro lets you bind some settings temporarily.  The first argument
+/// This macro lets you bind some [`Settings`](crate::Settings) temporarily.  The first argument
 /// takes key value pairs that should be set, the second is the block to
-/// execute.  All settings can be set (`sort_maps => value` maps roughly
-/// to `set_sort_maps(value)`).
+/// execute.  All settings can be set (`sort_maps => value` maps to `set_sort_maps(value)`).
+/// The exception are redactions which can only be set to a vector this way.
+///
+/// This example:
 ///
 /// ```rust
 /// insta::with_settings!({sort_maps => true}, {
 ///     // run snapshot test here
 /// });
 /// ```
+///
+/// Is equivalent to the following:
+///
+/// ```rust
+/// # use insta::Settings;
+/// let mut settings = Settings::clone_current();
+/// settings.set_sort_maps(true);
+/// settings.bind(|| {
+///     // run snapshot test here
+/// });
+/// ```
+///
+/// Note: before insta 0.17 this macro used
+/// [`Settings::new`](crate::Settings::new) which meant that original settings
+/// were always reset rather than extended.
 #[macro_export]
 macro_rules! with_settings {
     ({$($k:ident => $v:expr),*$(,)?}, $body:block) => {{
-        let mut settings = $crate::Settings::new();
+        let mut settings = $crate::Settings::clone_current();
         $(
-            settings._private_inner_mut().$k = $v.into();
+            settings._private_inner_mut().$k($v);
         )*
         settings.bind(|| $body)
     }}
@@ -421,8 +512,34 @@ macro_rules! with_settings {
 
 /// Executes a closure for all input files matching a glob.
 ///
-/// The closure is passed the path to the file.
+/// The closure is passed the path to the file.  You can use [`std::fs::read_to_string`]
+/// or similar functions to load the file and process it.
+///
+/// ```
+/// # use insta::{assert_snapshot, glob, Settings};
+/// # let mut settings = Settings::clone_current();
+/// # settings.set_allow_empty_glob(true);
+/// # let _dropguard = settings.bind_to_scope();
+/// use std::fs;
+///
+/// glob!("inputs/*.txt", |path| {
+///     let input = fs::read_to_string(path).unwrap();
+///     assert_snapshot!(input.to_uppercase());
+/// });
+/// ```
+///
+/// The `INSTA_GLOB_FILTER` environment variable can be set to only execute certain files.
+/// The format of the filter is a semicolon separated filter.  For instance by setting
+/// `INSTA_GLOB_FILTER` to `foo-*txt;bar-*.txt` only files starting with `foo-` or `bar-`
+/// end ending in `.txt` will be executed.  When using `cargo-insta` the `--glob-filter`
+/// option can be used instead.
+///
+/// Another effect of the globbing system is that snapshot failures within the glob macro
+/// are deferred until the end of of it.  In other words this means that each snapshot
+/// assertion within the `glob!` block are reported.  It can be disabled by setting
+/// `INSTA_GLOB_FAIL_FAST` environment variable to `1`.
 #[cfg(feature = "glob")]
+#[cfg_attr(docsrs, doc(cfg(feature = "glob")))]
 #[macro_export]
 macro_rules! glob {
     ($glob:expr, $closure:expr) => {{
