@@ -9,7 +9,7 @@ use crate::content::Content;
 pub struct SelectorParseError(pest::error::Error<Rule>);
 
 impl SelectorParseError {
-    /// Return the column of where the error ocurred.
+    /// Return the column of where the error occurred.
     pub fn column(&self) -> usize {
         match self.0.line_col {
             pest::error::LineColLocation::Pos((_, col)) => col,
@@ -23,6 +23,7 @@ impl SelectorParseError {
 /// This can be converted into a string with `to_string` to see a stringified
 /// path that the selector matched.
 #[derive(Clone, Debug)]
+#[cfg_attr(docsrs, doc(cfg(feature = "redactions")))]
 pub struct ContentPath<'a>(&'a [PathItem]);
 
 impl<'a> fmt::Display for ContentPath<'a> {
@@ -48,6 +49,7 @@ impl<'a> fmt::Display for ContentPath<'a> {
 /// Replaces a value with another one.
 
 /// Represents a redaction.
+#[cfg_attr(docsrs, doc(cfg(feature = "redactions")))]
 pub enum Redaction {
     /// Static redaction with new content.
     Static(Content),
@@ -121,6 +123,7 @@ impl<'a> From<&'a [u8]> for Redaction {
 ///     "[uuid]"
 /// }));
 /// ```
+#[cfg_attr(docsrs, doc(cfg(feature = "redactions")))]
 pub fn dynamic_redaction<I, F>(func: F) -> Redaction
 where
     I: Into<Content>,
@@ -129,6 +132,40 @@ where
     Redaction::Dynamic(Box::new(move |c, p| func(c, p).into()))
 }
 
+/// Creates a dynamic redaction that sorts the value at the selector.
+///
+/// This is useful to force something like a set or map to be ordered to make
+/// it deterministic.  This is necessary as insta's serialization support is
+/// based on serde which does not have native set support.  As a result vectors
+/// (which need to retain order) and sets (which should be given a stable order)
+/// look the same.
+///
+/// ```rust
+/// # use insta::{Settings, sorted_redaction};
+/// # let mut settings = Settings::new();
+/// settings.add_redaction(".flags", sorted_redaction());
+/// ```
+#[cfg_attr(docsrs, doc(cfg(feature = "redactions")))]
+pub fn sorted_redaction() -> Redaction {
+    fn sort(mut value: Content, _path: ContentPath) -> Content {
+        match value.resolve_inner_mut() {
+            Content::Seq(ref mut val) => {
+                val.sort_by(|a, b| a.partial_cmp(b).unwrap_or(std::cmp::Ordering::Equal))
+            }
+            Content::Map(ref mut val) => {
+                val.sort_by(|a, b| a.partial_cmp(b).unwrap_or(std::cmp::Ordering::Equal))
+            }
+            Content::Struct(_, ref mut fields)
+            | Content::StructVariant(_, _, _, ref mut fields) => {
+                fields.sort_by(|a, b| a.partial_cmp(b).unwrap_or(std::cmp::Ordering::Equal))
+            }
+            _ => {}
+        }
+        value
+    }
+    dynamic_redaction(sort)
+}
+
 impl Redaction {
     /// Performs the redaction of the value at the given path.
     fn redact(&self, value: Content, path: &[PathItem]) -> Content {
@@ -190,7 +227,7 @@ impl PathItem {
     }
 }
 
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub enum Segment<'a> {
     DeepWildcard,
     Wildcard,
@@ -316,7 +353,7 @@ impl<'a> Selector<'a> {
         match *segment {
             Segment::Wildcard => true,
             Segment::DeepWildcard => true,
-            Segment::Key(ref k) => element.as_str() == Some(&k),
+            Segment::Key(ref k) => element.as_str() == Some(k),
             Segment::Index(i) => element.as_u64() == Some(i),
             Segment::Range(start, end) => element.range_check(start, end),
         }
@@ -359,7 +396,7 @@ impl<'a> Selector<'a> {
 
     pub fn is_match(&self, path: &[PathItem]) -> bool {
         for selector in &self.selectors {
-            if self.selector_is_match(&selector, path) {
+            if self.selector_is_match(selector, path) {
                 return true;
             }
         }
@@ -410,17 +447,22 @@ impl<'a> Selector<'a> {
         redaction: &Redaction,
         path: &mut Vec<PathItem>,
     ) -> Content {
-        if self.is_match(&path) {
+        if self.is_match(path) {
             redaction.redact(value, path)
         } else {
             match value {
                 Content::Map(map) => Content::Map(
                     map.into_iter()
                         .map(|(key, value)| {
-                            path.push(PathItem::Content(key.clone()));
+                            path.push(PathItem::Field("$key"));
+                            let new_key = self.redact_impl(key.clone(), redaction, path);
+                            path.pop();
+
+                            path.push(PathItem::Content(key));
                             let new_value = self.redact_impl(value, redaction, path);
                             path.pop();
-                            (key, new_value)
+
+                            (new_key, new_value)
                         })
                         .collect(),
                 ),
@@ -469,6 +511,7 @@ impl<'a> Selector<'a> {
 
 #[test]
 fn test_range_checks() {
+    use similar_asserts::assert_eq;
     assert_eq!(PathItem::Index(0, 10).range_check(None, Some(-1)), true);
     assert_eq!(PathItem::Index(9, 10).range_check(None, Some(-1)), false);
     assert_eq!(PathItem::Index(0, 10).range_check(Some(1), Some(-1)), false);
