@@ -15,3 +15,11 @@ fn test_globs_follow_links() {
         insta::assert_json_snapshot!(&contents);
     });
 }
+
+#[test]
+#[should_panic(expected = "the glob! macro did not match any files.")]
+fn test_empty_glob_fails() {
+    insta::glob!("nonexistent", |_| {
+        // nothing
+    });
+}
