@@ -1,9 +1,18 @@
 #![cfg(feature = "redactions")]
 
 use insta::_macro_support::Selector;
-use insta::{
-    assert_debug_snapshot, assert_json_snapshot, assert_yaml_snapshot, with_settings, Settings,
-};
+#[cfg(feature = "csv")]
+use insta::assert_csv_snapshot;
+#[cfg(feature = "json")]
+use insta::assert_json_snapshot;
+#[cfg(feature = "ron")]
+use insta::assert_ron_snapshot;
+#[cfg(feature = "toml")]
+use insta::assert_toml_snapshot;
+#[cfg(feature = "yaml")]
+use insta::assert_yaml_snapshot;
+
+use insta::assert_debug_snapshot;
 use serde::Serialize;
 
 #[test]
@@ -34,6 +43,7 @@ pub struct User {
     extra: String,
 }
 
+#[cfg(feature = "yaml")]
 #[test]
 fn test_with_random_value() {
     assert_yaml_snapshot!("user", &User {
@@ -46,6 +56,7 @@ fn test_with_random_value() {
     });
 }
 
+#[cfg(feature = "yaml")]
 #[test]
 fn test_with_random_value_inline_callback() {
     assert_yaml_snapshot!("user", &User {
@@ -55,13 +66,14 @@ fn test_with_random_value_inline_callback() {
         extra: "".to_string(),
     }, {
         ".id" => insta::dynamic_redaction(|value, path| {
-            assert_eq!(path.to_string(), ".id");
-            assert_eq!(value.as_u64().unwrap(), 23);
+            similar_asserts::assert_eq!(path.to_string(), ".id");
+            similar_asserts::assert_eq!(value.as_u64().unwrap(), 23);
             "[id]"
         }),
     });
 }
 
+#[cfg(feature = "yaml")]
 #[test]
 fn test_with_random_value_and_trailing_comma() {
     assert_yaml_snapshot!("user", &User {
@@ -77,7 +89,7 @@ fn test_with_random_value_and_trailing_comma() {
 #[cfg(feature = "csv")]
 #[test]
 fn test_with_random_value_csv() {
-    insta::assert_csv_snapshot!("user_csv", &User {
+    assert_csv_snapshot!("user_csv", &User {
         id: 44,
         username: "julius_csv".to_string(),
         email: Email("julius@example.com".to_string()),
@@ -90,7 +102,7 @@ fn test_with_random_value_csv() {
 #[cfg(feature = "ron")]
 #[test]
 fn test_with_random_value_ron() {
-    insta::assert_ron_snapshot!("user_ron", &User {
+    assert_ron_snapshot!("user_ron", &User {
         id: 53,
         username: "john_ron".to_string(),
         email: Email("john@example.com".to_string()),
@@ -103,7 +115,7 @@ fn test_with_random_value_ron() {
 #[cfg(feature = "toml")]
 #[test]
 fn test_with_random_value_toml() {
-    insta::assert_toml_snapshot!("user_toml", &User {
+    assert_toml_snapshot!("user_toml", &User {
         id: 53,
         username: "john_ron".to_string(),
         email: Email("john@example.com".to_string()),
@@ -113,6 +125,7 @@ fn test_with_random_value_toml() {
     });
 }
 
+#[cfg(feature = "json")]
 #[test]
 fn test_with_random_value_json() {
     assert_json_snapshot!("user_json", &User {
@@ -126,9 +139,10 @@ fn test_with_random_value_json() {
     });
 }
 
+#[cfg(feature = "json")]
 #[test]
 fn test_with_random_value_json_settings() {
-    let mut settings = Settings::new();
+    let mut settings = insta::Settings::new();
     settings.add_redaction(".id", "[id]");
     settings.add_redaction(".extra", "[extra]");
     settings.bind(|| {
@@ -144,12 +158,13 @@ fn test_with_random_value_json_settings() {
     });
 }
 
+#[cfg(feature = "json")]
 #[test]
 fn test_with_callbacks() {
-    let mut settings = Settings::new();
+    let mut settings = insta::Settings::new();
     settings.add_dynamic_redaction(".id", |value, path| {
-        assert_eq!(path.to_string(), ".id");
-        assert_eq!(value.as_u64().unwrap(), 1234);
+        similar_asserts::assert_eq!(path.to_string(), ".id");
+        similar_asserts::assert_eq!(value.as_u64().unwrap(), 1234);
         "[id]"
     });
     settings.bind(|| {
@@ -165,9 +180,10 @@ fn test_with_callbacks() {
     });
 }
 
+#[cfg(feature = "json")]
 #[test]
 fn test_with_random_value_json_settings2() {
-    with_settings!({redactions => vec![
+    insta::with_settings!({redactions => vec![
         (".id", "[id]".into()),
         (".extra", "[extra]".into()),
     ]}, {
@@ -182,6 +198,7 @@ fn test_with_random_value_json_settings2() {
     });
 }
 
+#[cfg(feature = "json")]
 #[test]
 fn test_redact_newtype_struct() {
     #[derive(Serialize)]
@@ -206,6 +223,7 @@ fn test_redact_newtype_struct() {
     "###);
 }
 
+#[cfg(feature = "yaml")]
 #[test]
 fn test_redact_newtype_enum() {
     #[derive(Serialize)]
@@ -245,6 +263,7 @@ fn test_redact_newtype_enum() {
     "###);
 }
 
+#[cfg(feature = "json")]
 #[test]
 fn test_redact_recursive() {
     #[derive(Serialize)]
@@ -270,3 +289,128 @@ fn test_redact_recursive() {
     }
     "###);
 }
+
+#[cfg(feature = "yaml")]
+#[test]
+fn test_struct_array_redaction() {
+    #[derive(Serialize)]
+    pub struct Product {
+        _id: String,
+        product_name: String,
+    }
+
+    #[derive(Serialize)]
+    pub struct Checkout {
+        _id: String,
+        products: Vec<Product>,
+    }
+
+    let checkout = Checkout {
+        _id: "checkout/1".to_string(),
+        products: vec![
+            Product {
+                _id: "product/1".to_string(),
+                product_name: "a car".to_string(),
+            },
+            Product {
+                _id: "product/2".to_string(),
+                product_name: "a boat".to_string(),
+            },
+        ],
+    };
+
+    assert_yaml_snapshot!(vec![checkout], {
+        "[]._id" => "[checkout_id]",
+        "[].products[]._id" => "[product_id]",
+        "[].products[].product_name" => "[product_name]",
+    });
+}
+
+#[cfg(feature = "yaml")]
+#[test]
+fn test_map_key_redaction() {
+    #[derive(Serialize, Hash, PartialEq, PartialOrd, Eq, Ord)]
+    struct Key {
+        bucket: u32,
+        value: u32,
+    }
+
+    #[derive(Serialize)]
+    struct Foo {
+        hm: std::collections::HashMap<Key, u32>,
+        btm: std::collections::BTreeMap<(u32, u32), u32>,
+    }
+
+    let mut hm = std::collections::HashMap::new();
+    hm.insert(
+        Key {
+            bucket: 1,
+            value: 0,
+        },
+        42,
+    );
+    let mut btm = std::collections::BTreeMap::new();
+    btm.insert((0, 0), 23);
+    let foo_value = Foo { hm, btm };
+
+    assert_yaml_snapshot!(foo_value, {
+        ".hm.$key.bucket" => "[bucket]",
+        ".btm.$key" => "[key]",
+    });
+}
+
+#[cfg(feature = "json")]
+#[test]
+fn test_ordering() {
+    #[derive(Debug, Serialize)]
+    pub struct User {
+        id: u64,
+        username: String,
+        flags: std::collections::HashSet<String>,
+    }
+
+    let mut settings = insta::Settings::new();
+    settings.add_redaction(".id", "[id]");
+    settings.sort_selector(".flags");
+    settings.bind(|| {
+        assert_json_snapshot!(
+            "user_json_flags",
+            &User {
+                id: 122,
+                username: "jason_doe".to_string(),
+                flags: vec!["zzz".into(), "foo".into(), "aha".into(), "is_admin".into()]
+                    .into_iter()
+                    .collect(),
+            }
+        );
+    });
+}
+
+#[cfg(feature = "json")]
+#[test]
+fn test_ordering_newtype_set() {
+    #[derive(Debug, Serialize)]
+    pub struct MySet(std::collections::HashSet<String>);
+
+    #[derive(Debug, Serialize)]
+    pub struct User {
+        id: u64,
+        username: String,
+        flags: MySet,
+    }
+
+    assert_json_snapshot!(
+        "user_json_flags_alt",
+        &User {
+            id: 122,
+            username: "jason_doe".to_string(),
+            flags: MySet(vec!["zzz".into(), "foo".into(), "aha".into(), "is_admin".into()]
+                .into_iter()
+                .collect()),
+        },
+        {
+            "." => insta::sorted_redaction(),
+            ".flags" => insta::sorted_redaction()
+        }
+    );
+}
